@@ -10,13 +10,12 @@
 */
 #include "datasourcefactory.h"
 #include "bitset.h"
-#include "kiodownload.h"
 #include "settings.h"
 
 #include "core/kget.h"
 #include "core/verifier.h"
 
-#include <math.h>
+#include <cmath>
 
 #include <QtCore/QDir>
 #include <QtCore/QTimer>
@@ -38,6 +37,7 @@ const int SPEEDTIMER = 1000;//1 second...
 
 DataSourceFactory::DataSourceFactory(QObject *parent, const KUrl &dest, KIO::filesize_t size, KIO::fileoffset_t segSize)
   : QObject(parent),
+    m_capabilities(0),
     m_dest(dest),
     m_size(size),
     m_downloadedSize(0),
@@ -51,6 +51,7 @@ DataSourceFactory::DataSourceFactory(QObject *parent, const KUrl &dest, KIO::fil
     m_open(false),
     m_blocked(false),
     m_startTried(false),
+    m_findFilesizeTried(false),
     m_assignTried(false),
     m_movingFile(false),
     m_finished(false),
@@ -58,7 +59,6 @@ DataSourceFactory::DataSourceFactory(QObject *parent, const KUrl &dest, KIO::fil
     m_sizeInitiallyDefined(m_size),
     m_maxMirrorsUsed(3),
     m_speedTimer(0),
-    m_tempDownload(0),
     m_status(Job::Stopped),
     m_statusBeforeMove(m_status),
     m_verifier(0),
@@ -121,90 +121,49 @@ void DataSourceFactory::deinit()
 
 void DataSourceFactory::findFileSize()
 {
-    kDebug(5001) << "Find the filesize";
+    kDebug(5001) << "Find the filesize" << this;
     if (!m_size && !m_dest.isEmpty() && !m_sources.isEmpty()) {
-        //FIXME 4.5 move findingFileSize to the TransferDataSources themselves (make it a capability
-        //that they can support); no need for checking the protocol etc. in 4.4 as there is no other TransferDataSource than the one from MultiSegKIO for downloading yet
-        if (!m_tempDownload) {
-            m_tempDownload = new KioDownload(m_sources.begin().value()->sourceUrl(), m_dest, this);
-            connect(m_tempDownload, SIGNAL(processedSize(KIO::filesize_t)), this, SLOT(slotKIOProcessedSize(KIO::filesize_t)));
-            connect(m_tempDownload, SIGNAL(percent(ulong)), this, SIGNAL(percent(ulong)));
-            connect(m_tempDownload, SIGNAL(speed(ulong)), this, SIGNAL(speed(ulong)));
-            connect(m_tempDownload, SIGNAL(totalSize(KIO::filesize_t)), this, SLOT(sizeFound(KIO::filesize_t)));
-            connect(m_tempDownload, SIGNAL(finished()), this, SLOT(finished()));
-            connect(m_tempDownload, SIGNAL(error()), this, SLOT(slotKIOError()));
+        foreach (TransferDataSource *source, m_sources) {
+            if (source->capabilities() & Transfer::Cap_FindFilesize) {
+                connect(source, SIGNAL(foundFileSize(TransferDataSource*,KIO::filesize_t,QPair<int,int>)), this, SLOT(slotFoundFileSize(TransferDataSource*,KIO::filesize_t,QPair<int,int>)));
+                connect(source, SIGNAL(finishedDownload(TransferDataSource*,KIO::filesize_t)), this, SLOT(slotFinishedDownload(TransferDataSource*,KIO::filesize_t)));
+
+                m_speedTimer->start();
+                source->findFileSize(m_segSize);
+                changeStatus(Job::Running);
+                slotUpdateCapabilities();
+                return;
+            }
         }
-        m_tempDownload->start();
-        changeStatus(Job::Running);
     }
 }
 
-void DataSourceFactory::slotKIOProcessedSize(KIO::filesize_t size)
+void DataSourceFactory::slotFoundFileSize(TransferDataSource *source, KIO::filesize_t fileSize, const QPair<int, int> &segmentRange)
 {
-    m_downloadedSize = size;
-    emit processedSize(m_downloadedSize);
-}
-void DataSourceFactory::slotKIOError()
-{
-    m_size = 0; //if KIO quits with an error any already gotten size should be ignored
-}
-
-void DataSourceFactory::sizeFound(KIO::filesize_t size)
-{
-    m_size = size;
-    kDebug(5001) << "Size found: " << m_size;
+    m_size = fileSize;
+    kDebug(5001) << source << "found size" << m_size << "and is assigned segments" << segmentRange << this;
     emit totalSize(m_size);
 
-    if (!m_tempDownload->isResumable()) {
-        kDebug(5001) << "Download not resumeable, so do not stop it.";
-        return;
-    }
-
     init();
 
-    //use what the tempDownload has downloaded if it is larger than a segment
-    m_tempDownload->stop();
-    const quint32 segmentsDownloaded = m_tempDownload->processedSize() / m_segSize;
-    if (segmentsDownloaded)
-    {
-        kDebug(5001) << "Segments reused of the tempDownload: " << segmentsDownloaded;
-
-        m_downloadedSize = segmentsDownloaded * m_segSize;
-        for (quint32 i = 0; i < segmentsDownloaded; ++i)
-        {
-            m_startedChunks->set(i, true);
-            m_finishedChunks->set(i, true);
-        }
+    if ((segmentRange.first != -1) && (segmentRange.second != -1)) {
+        m_startedChunks->setRange(segmentRange.first, segmentRange.second, true);
     }
 
-    delete m_tempDownload;
-    m_tempDownload = 0;
-
-    if (m_startTried)
-    {
+    if (m_startTried) {
         start();
     }
 }
 
-void DataSourceFactory::finished()
+void DataSourceFactory::slotFinishedDownload(TransferDataSource *source, KIO::filesize_t size)
 {
-    m_downloadedSize = m_tempDownload->processedSize();
+    Q_UNUSED(source)
+    Q_UNUSED(size)
 
-    delete m_tempDownload;
-    m_tempDownload = 0;
-
-    changeStatus(Job::Finished);
-
-    //set all chunks to true, that is useful for saving
-    init();
-    if (m_startedChunks) {
-        m_startedChunks->setAll(true);
-    }
-    if (m_finishedChunks) {
-        m_finishedChunks->setAll(true);
-    }
+    m_speedTimer->stop();
+    m_finished = true;
 }
-
+//FIXME is this even needed
 bool DataSourceFactory::checkLocalFile()
 {
     QString dest_orig = m_dest.toLocalFile();
@@ -250,9 +209,10 @@ void DataSourceFactory::start()
     m_downloadInitialized = true;
 
     if (checkLocalFile()) {
-        if (!m_size) {
+        if (!m_putJob) {
+            m_putJob = KIO::open(m_dest, QIODevice::WriteOnly | QIODevice::ReadOnly);
+            connect(m_putJob, SIGNAL(open(KIO::Job*)), this, SLOT(open(KIO::Job*)));
             m_startTried = true;
-            findFileSize();
             return;
         }
     } else {
@@ -264,6 +224,20 @@ void DataSourceFactory::start()
 
     init();
 
+    if (!m_open) {
+        m_startTried = true;
+        return;
+    }
+
+    if (!m_size) {
+        if (!m_findFilesizeTried && m_sources.count()) {
+            m_findFilesizeTried = true;
+            findFileSize();
+        }
+        m_startTried = true;
+        return;
+    }
+
     if (assignNeeded()) {
         if (m_sources.count()) {
             kDebug(5001) << "Assigning a TransferDataSource.";
@@ -279,41 +253,29 @@ void DataSourceFactory::start()
     if (m_assignTried) {
         m_assignTried = false;
 
-        //TODO maybe remove this later, if loading works without it
-        if (!m_startedChunks->allOn()) {
-            foreach(TransferDataSource *source, m_sources) {
-                assignSegments(source);
-            }
+        foreach(TransferDataSource *source, m_sources) {
+            assignSegments(source);
         }
     }
 
-    if (checkLocalFile())
-    {
-        if (!m_putJob)
-        {
-            QFile::resize(m_dest.pathOrUrl(), m_size);//TODO should we keep that?
-            m_putJob = KIO::open(m_dest, QIODevice::WriteOnly | QIODevice::ReadOnly);
-            connect(m_putJob, SIGNAL(open(KIO::Job*)), this, SLOT(open(KIO::Job*)));
-            m_startTried = true;
-            return;
-        }
-        if (m_open) {
-            m_speedTimer->start();
+    if (m_open) {
+        QFile::resize(m_dest.pathOrUrl(), m_size);//TODO should we keep that?
+        m_speedTimer->start();
 
-            foreach (TransferDataSource *source, m_sources) {
-                source->start();
-            }
-
-            m_startTried = false;
-            changeStatus(Job::Running);
+        foreach (TransferDataSource *source, m_sources) {
+            source->start();
         }
+
+        m_startTried = false;
+        changeStatus(Job::Running);
     }
+    slotUpdateCapabilities();
 }
 
 void DataSourceFactory::open(KIO::Job *job)
 {
     Q_UNUSED(job)
-    kDebug(5001) << "File opened.";
+    kDebug(5001) << "File opened" << this;
 
     if (!m_speedTimer)
     {
@@ -332,7 +294,7 @@ void DataSourceFactory::open(KIO::Job *job)
 
 void DataSourceFactory::stop()
 {
-    kDebug(5001) << "Stopping";
+    kDebug(5001) << "Stopping" << this;
     if (m_movingFile || (m_status == Job::Finished))
     {
         return;
@@ -342,14 +304,15 @@ void DataSourceFactory::stop()
     {
         m_speedTimer->stop();
     }
-    if (m_tempDownload) {
-        m_tempDownload->stop();
-    }
+
     foreach (TransferDataSource *source, m_sources) {
         source->stop();
     }
     m_startTried = false;
+    m_findFilesizeTried = false;
     changeStatus(Job::Stopped);
+
+    slotUpdateCapabilities();
 }
 
 void DataSourceFactory::setDoDownload(bool doDownload)
@@ -417,12 +380,8 @@ void DataSourceFactory::addMirror(const KUrl &url, bool used, int numParalellCon
                     kDebug(5001) << "Remove connection with segments" << removed;
                     const int start = removed.first;
                     const int end = removed.second;
-                    if ((start != -1) && (end != -1))
-                    {
-                        for (int k = start; k <= end; ++k)
-                        {
-                            m_startedChunks->set(k, false);
-                        }
+                    if ((start != -1) && (end != -1)) {
+                        m_startedChunks->setRange(start, end, false);
                     }
                 }
             }
@@ -434,7 +393,7 @@ void DataSourceFactory::addMirror(const KUrl &url, bool used, int numParalellCon
                 TransferDataSource *source = KGet::createTransferDataSource(url, QDomElement(), this);
                 if (source)
                 {
-                    kDebug(5001) << "Successfully created a TransferDataSource for " << url.pathOrUrl();
+                    kDebug(5001) << "Successfully created a TransferDataSource for " << url.pathOrUrl() << this;
 
                     //url might have been an unused Mirror, so remove it in any case
                     const int index = m_unusedUrls.indexOf(url);
@@ -450,12 +409,15 @@ void DataSourceFactory::addMirror(const KUrl &url, bool used, int numParalellCon
                         source->setSupposedSize(m_size);
                     }
 
+                    connect(source, SIGNAL(capabilitiesChanged()), this, SLOT(slotUpdateCapabilities()));
                     connect(source, SIGNAL(brokenSegments(TransferDataSource*,QPair<int, int>)), this, SLOT(brokenSegments(TransferDataSource*,QPair<int, int>)));
                     connect(source, SIGNAL(broken(TransferDataSource*, TransferDataSource::Error)), this, SLOT(broken(TransferDataSource*,TransferDataSource::Error)));
                     connect(source, SIGNAL(finishedSegment(TransferDataSource*,int,bool)), this, SLOT(finishedSegment(TransferDataSource*,int,bool)));
                     connect(source, SIGNAL(data(KIO::fileoffset_t, const QByteArray&, bool&)), this, SLOT(slotWriteData(KIO::fileoffset_t, const QByteArray&, bool&)));
                     connect(source, SIGNAL(freeSegments(TransferDataSource*,QPair<int,int>)), this, SLOT(slotFreeSegments(TransferDataSource*,QPair<int,int>)));
 
+                    slotUpdateCapabilities();
+
                     assignSegments(source);
 
                     //the job is already running, so also start the TransferDataSource
@@ -514,13 +476,9 @@ void DataSourceFactory::removeMirror(const KUrl &url)
         {
             const int start = assigned[i].first;
             const int end = assigned[i].second;
-            if ((start != -1) && (end != -1))
-            {
-                for (int k = start; k <= end; ++k)
-                {
-                    kDebug(5001) << "Segment" << k << "not assigned anymore.";
-                    m_startedChunks->set(k, false);
-                }
+            if ((start != -1) && (end != -1)) {
+                m_startedChunks->setRange(start, end, false);
+                kDebug(5001) << "Segmentrange" << start << '-' << end << "not assigned anymore.";
             }
         }
     }
@@ -597,7 +555,7 @@ bool DataSourceFactory::assignNeeded() const
 void DataSourceFactory::brokenSegments(TransferDataSource *source, const QPair<int, int> &segmentRange)
 {
     kDebug(5001) << "Segments" << segmentRange << "broken," << source;
-    if (!source || (segmentRange.first < 0) || (segmentRange.second < 0) || (static_cast<quint32>(segmentRange.second) > m_finishedChunks->getNumBits()))
+    if (!source || !m_startedChunks || !m_finishedChunks || (segmentRange.first < 0) || (segmentRange.second < 0) || (static_cast<quint32>(segmentRange.second) > m_finishedChunks->getNumBits()))
     {
         return;
     }
@@ -605,9 +563,7 @@ void DataSourceFactory::brokenSegments(TransferDataSource *source, const QPair<i
     const int start = segmentRange.first;
     const int end = segmentRange.second;
     if ((start != -1) && (end != -1)) {
-        for (int k = start; k <= end; ++k) {
-            m_startedChunks->set(k, false);
-        }
+        m_startedChunks->setRange(start, end, false);
     }
 
     removeMirror(source->sourceUrl());
@@ -634,10 +590,8 @@ void DataSourceFactory::slotFreeSegments(TransferDataSource *source, QPair<int,
     const int start = segmentRange.first;
     const int end = segmentRange.second;
     if ((start != -1) && (end != -1)) {
-        for (int i = start; i <= end; ++i) {
-            kDebug(5001) << "Segment" << i << "not assigned anymore.";
-            m_startedChunks->set(i, false);
-        }
+        m_startedChunks->setRange(start, end, false);
+        kDebug(5001) << "Segmentrange" << start << '-' << end << "not assigned anymore.";
     }
 
     assignSegments(source);
@@ -727,10 +681,7 @@ void DataSourceFactory::assignSegments(TransferDataSource *source)
     const KIO::fileoffset_t lastSegSize = ((static_cast<uint>(newEnd + 1) == m_startedChunks->getNumBits() && rest) ? rest : m_segSize);
 
     kDebug(5001) << "Segments assigned:" << newStart << "-" << newEnd << "segment-size:" << m_segSize << "rest:" << rest;
-
-    for (int i = newStart; i <= newEnd; ++i) {
-        m_startedChunks->set(i, true);
-    }
+    m_startedChunks->setRange(newStart, newEnd, true);
     source->addSegments(qMakePair(m_segSize, lastSegSize), qMakePair(newStart, newEnd));
 
     //there should still be segments added to this transfer
@@ -798,7 +749,9 @@ void DataSourceFactory::speedChanged()
         m_prevDownloadedSizes.removeFirst();
 
     emit speed(m_speed);
-    emit percent(m_downloadedSize * 100 / m_size);
+    if (m_size) {
+        emit percent(m_downloadedSize * 100 / m_size);
+    }
 }
 
 void DataSourceFactory::killPutJob()
@@ -886,12 +839,17 @@ void DataSourceFactory::repair()
     }
 
     m_finished = false;
+
+    connect(verifier(), SIGNAL(brokenPieces(QList<KIO::fileoffset_t>,KIO::filesize_t)), this, SLOT(slotRepair(QList<KIO::fileoffset_t>,KIO::filesize_t)));
+
     verifier()->brokenPieces();
 }
 
-void DataSourceFactory::slotRepair(const QList<QPair<KIO::fileoffset_t, KIO::filesize_t> > &brokenPieces)
+void DataSourceFactory::slotRepair(const QList<KIO::fileoffset_t> &offsets, KIO::filesize_t length)
 {
-    if (brokenPieces.isEmpty())
+    disconnect(verifier(), SIGNAL(brokenPieces(QList<KIO::fileoffset_t>,KIO::filesize_t)), this, SLOT(slotRepair(QList<KIO::fileoffset_t>,KIO::filesize_t)));
+
+    if (offsets.isEmpty())
     {
         kDebug(5001) << "Redownload everything";
         m_startedChunks->clear();
@@ -900,17 +858,11 @@ void DataSourceFactory::slotRepair(const QList<QPair<KIO::fileoffset_t, KIO::fil
     else
     {
         kDebug(5001) << "Redownload broken pieces";
-        QList<QPair<KIO::fileoffset_t, KIO::filesize_t> >::const_iterator it;
-        QList<QPair<KIO::fileoffset_t, KIO::filesize_t> >::const_iterator itEnd = brokenPieces.constEnd();
-        for (it = brokenPieces.constBegin(); it != itEnd; ++it)
-        {
-            const quint32 startSegment = (*it).first / m_segSize;
-            const quint32 endSegment = ceil((*it).second / static_cast<double>(m_segSize)) - 1 + startSegment;
-            for (quint32 i = startSegment; i <= endSegment; ++i)
-            {
-                m_startedChunks->set(i, false);
-                m_finishedChunks->set(i, false);
-            }
+        for (int i = 0; i < offsets.count(); ++i) {
+            const int start = offsets[i] / m_segSize;
+            const int end = std::ceil(length / static_cast<double>(m_segSize)) - 1 + start;
+            m_startedChunks->setRange(start, end, false);
+            m_finishedChunks->setRange(start, end, false);
         }
     }
 
@@ -1075,6 +1027,8 @@ void DataSourceFactory::changeStatus(Job::Status status, bool loaded)
                     signature()->verify();
                 }
             }
+
+            slotUpdateCapabilities();
             break;
         default:
             //TODO handle Delayed
@@ -1257,7 +1211,6 @@ Verifier *DataSourceFactory::verifier()
 {
     if (!m_verifier) {
         m_verifier = new Verifier(m_dest, this);
-        connect(m_verifier, SIGNAL(brokenPieces(QList<QPair<KIO::fileoffset_t,KIO::filesize_t> >)), this, SLOT(slotRepair(QList<QPair<KIO::fileoffset_t,KIO::filesize_t> >)));
     }
 
     return m_verifier;
@@ -1272,4 +1225,36 @@ Signature *DataSourceFactory::signature()
     return m_signature;
 }
 
+Transfer::Capabilities DataSourceFactory::capabilities() const
+{
+    return m_capabilities;
+}
+
+void DataSourceFactory::slotUpdateCapabilities()
+{
+    const Transfer::Capabilities oldCaps = capabilities();
+    Transfer::Capabilities newCaps = 0;
+
+    foreach (TransferDataSource *source, m_sources) {
+        if (!source->assignedSegments().isEmpty()) {
+            if (newCaps) {
+                newCaps &= source->capabilities();
+            } else {
+                newCaps = source->capabilities();
+            }
+        }
+    }
+
+    if ((newCaps & Transfer::Cap_Resuming) || (status() == Job::Finished) || (status() == Job::Stopped)) {
+        newCaps |= Transfer::Cap_Moving | Transfer::Cap_Renaming;
+    }
+
+    newCaps |= Transfer::Cap_MultipleMirrors;
+
+    if (oldCaps != newCaps) {
+        m_capabilities = newCaps;
+        emit capabilitiesChanged();
+    }
+}
+
 #include "datasourcefactory.moc"
