@@ -49,6 +49,11 @@ class KGET_EXPORT DataSourceFactory : public QObject
 
         ~DataSourceFactory();
 
+        /**
+         * The capabilities the DataSourceFactory supports
+         */
+        Transfer::Capabilities capabilities() const;
+
         /**
          * Deletes the created (downloadInitialized() is true) file if the download was not finished
          * Does not delete anything if the download never got started
@@ -163,6 +168,7 @@ class KGET_EXPORT DataSourceFactory : public QObject
         Signature *signature();
 
     Q_SIGNALS:
+        void capabilitiesChanged();
         void processedSize(KIO::filesize_t size);
         void totalSize(KIO::filesize_t size);
         void speed(ulong speed);
@@ -174,16 +180,15 @@ class KGET_EXPORT DataSourceFactory : public QObject
         void load(const QDomElement *e);
 
     private slots:
+        void slotUpdateCapabilities();
+
         /**
          * Tries to find the size of the file, automatically called
          * by start if no file size has been specified
          */
         void findFileSize();
-        /**
-         * Called from KIODownload when trying to find the filesize//TODO remove in 4.5 and combine it with TansferDataSource
-         */
-        void slotKIOProcessedSize(KIO::filesize_t size);
-        void slotKIOError();
+
+        void slotFoundFileSize(TransferDataSource *source, KIO::filesize_t fileSize, const QPair<int,int> &segmentRange);
 
         void assignSegments(TransferDataSource *source);
         /**
@@ -207,15 +212,15 @@ class KGET_EXPORT DataSourceFactory : public QObject
         void slotPercent(KJob *job, ulong percent);
         void open(KIO::Job *job);
         void speedChanged();
-        void sizeFound(KIO::filesize_t size);
-        void finished();
         /**
          * Kills the putjob and starts the moving of files
          */
         void startMove();
         void newDestResult(KJob *job);
 
-        void slotRepair(const QList<QPair<KIO::fileoffset_t,KIO::filesize_t> > &brokenPieces);
+        void slotRepair(const QList<KIO::fileoffset_t> &offsets, KIO::filesize_t length);
+
+        void slotFinishedDownload(TransferDataSource *source, KIO::filesize_t size);
 
     private:
         /**
@@ -239,6 +244,7 @@ class KGET_EXPORT DataSourceFactory : public QObject
         void changeStatus(Job::Status status, bool loaded = false);
 
     private:
+        Transfer::Capabilities m_capabilities;
         KUrl m_dest;
         KUrl m_newDest;
         KIO::filesize_t m_size;
@@ -268,6 +274,7 @@ class KGET_EXPORT DataSourceFactory : public QObject
          * start() could be recalled
          */
         bool m_startTried;
+        bool m_findFilesizeTried;
 
         bool m_assignTried;
         bool m_movingFile;
@@ -290,7 +297,6 @@ class KGET_EXPORT DataSourceFactory : public QObject
         QList<KUrl> m_unusedUrls;
         QList<int> m_unusedConnections;
         QTimer *m_speedTimer;
-        KioDownload *m_tempDownload;
         Job::Status m_status;
         Job::Status m_statusBeforeMove;
 
