@@ -27,8 +27,14 @@ class KGET_EXPORT LinkImporter : public QThread
 Q_OBJECT
 public:
     LinkImporter(const KUrl &source, QObject *parent);
+    LinkImporter(QObject *parent);
     ~LinkImporter();
 
+    /**
+    * Check for urls in clipboard
+    */
+    void checkClipboard(const QString &clipboardContent);
+
     /**
     * Start reading the url contents
     */
@@ -54,6 +60,12 @@ signals:
 private slots:
     void slotReadFile(const QUrl &url);
 
+private:
+    /**
+    * Checks if an url is valid and adds it to the transfers lists
+    */
+    void addTransfer(QString &link);
+
 private:
     KUrl m_url;
     QList <QString> m_transfers;
