@@ -31,7 +31,7 @@ TransferGroup::TransferGroup(TransferTreeModel * model, Scheduler * parent, cons
       m_percent(0), m_downloadSpeed(0), m_uploadSpeed(0),
       m_downloadLimit(0), m_uploadLimit(0),
       m_visibleDownloadLimit(0), m_visibleUploadLimit(0),
-      m_iconName("bookmark-new-list"), m_defaultFolder(0)
+      m_iconName("bookmark-new-list"), m_defaultFolder()
 {
     m_handler = new TransferGroupHandler(parent, this);
 }
@@ -66,14 +66,14 @@ int TransferGroup::uploadSpeed()
 
 bool TransferGroup::supportsSpeedLimits()
 {
-    bool b = true;
-    foreach (Job * job, runningJobs())
-    {
+    foreach (Job * job, runningJobs()) {
         Transfer * transfer = static_cast<Transfer*>(job);
-        if (!transfer->supportsSpeedLimits())
-            b = false;
+        if (!(transfer->capabilities() & Transfer::Cap_SpeedLimit)) {
+            return false;
+        }
     }
-    return b;
+
+    return true;
 }
 
 void TransferGroup::setStatus(Status queueStatus)
