@@ -5,7 +5,7 @@
    Copyright (C) 2002 Carsten Pfeiffer <pfeiffer@kde.org>
    Copyright (C) 2006 - 2008 Urs Wolfer <uwolfer @ kde.org>
    Copyright (C) 2006 Dario Massarin <nekkar@libero.it>
-   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>
+   Copyright (C) 2008 - 2009 Lukas Appelhans <l.appelhans@gmx.de>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -18,8 +18,8 @@
 #include "core/kget.h"
 #include "core/transferhandler.h"
 #include "core/transfergrouphandler.h"
+#include "core/transfertreemodel.h"
 #include "core/transfertreeselectionmodel.h"
-#include "dbus/dbusmodelobserver.h"
 #include "settings.h"
 #include "conf/preferencesdialog.h"
 #include "ui/viewscontainer.h"
@@ -30,7 +30,11 @@
 #include "ui/groupsettingsdialog.h"
 #include "ui/transfersettingsdialog.h"
 #include "ui/linkview/kget_linkview.h"
+#include "ui/metalinkcreator/metalinkcreator.h"
 #include "extensions/webinterface/httpserver.h"
+#ifdef DEBUG
+    #include "tests/testkget.h"
+#endif
 
 #include <kapplication.h>
 #include <kstandarddirs.h>
@@ -49,15 +53,17 @@
 #include <kactionmenu.h>
 #include <krun.h>
 #include <kicondialog.h>
-
+#include "core/verifier.h"
 #include <QClipboard>
 #include <QTimer>
+#include <QtTest/QtTest>
 
-MainWindow::MainWindow(bool showMainwindow, bool startWithoutAnimation, QWidget *parent)
+MainWindow::MainWindow(bool showMainwindow, bool startWithoutAnimation, bool doTesting, QWidget *parent)
     : KXmlGuiWindow( parent ),
       m_drop(0),
       m_dock(0),
       m_startWithoutAnimation(startWithoutAnimation),
+      m_doTesting(doTesting),
       m_webinterface(0)
 {
     // do not quit the app when it has been minimized to system tray and a new transfer dialog
@@ -66,7 +72,7 @@ MainWindow::MainWindow(bool showMainwindow, bool startWithoutAnimation, QWidget
     setAttribute(Qt::WA_DeleteOnClose, false);
 
     // create the model
-    KGet::self( this );
+    m_kget = KGet::self( this );
 
     // create actions
     setupActions();
@@ -74,8 +80,6 @@ MainWindow::MainWindow(bool showMainwindow, bool startWithoutAnimation, QWidget
     createGUI("kgetui.rc");
 
     m_viewsContainer = new ViewsContainer(this);
-    // initialize the model observer to export percents over dbus
-    m_dbusModelObserver = new DBusModelObserver();
 
     setCentralWidget(m_viewsContainer);
 
@@ -91,15 +95,14 @@ MainWindow::MainWindow(bool showMainwindow, bool startWithoutAnimation, QWidget
 
     //Some of the widgets are initialized in slotDelayedInit()
     QTimer::singleShot( 0, this, SLOT(slotDelayedInit()) );
-
-    // Update the title with the active transfers percent using the mainwindowmodelobserver
-    KGet::addObserver(new MainWindowModelObserver(this));
 }
 
 MainWindow::~MainWindow()
 {
     //Save the user's transfers
     KGet::save();
+    
+    NewTransferDialog::del();
 
     slotSaveMyself();
     // reset konqueror integration (necessary if user enabled / disabled temporarily integration from tray)
@@ -107,7 +110,8 @@ MainWindow::~MainWindow()
     // the following call saves options set in above dtors
     Settings::self()->writeConfig();
 
-    KGet::deleteSelf();
+    delete m_drop;
+    delete m_kget;
 }
 
 QSize MainWindow::sizeHint() const
@@ -178,6 +182,10 @@ void MainWindow::setupActions()
     exportPlainAction->setShortcuts(KShortcut("Ctrl+P"));
     connect(exportPlainAction, SIGNAL(triggered()), SLOT(slotExportPlainTransfers()));
 
+    KAction *createMetalinkAction = actionCollection()->addAction("create_metalink");
+    createMetalinkAction->setText(i18n("&Create a Metalink"));
+    connect(createMetalinkAction, SIGNAL(triggered()), SLOT(slotCreateMetalink()));
+
     KAction *deleteGroupAction = actionCollection()->addAction("delete_groups");
     deleteGroupAction->setText(i18n("Delete Group"));
     deleteGroupAction->setIcon(KIcon("edit-delete"));
@@ -221,13 +229,13 @@ void MainWindow::setupActions()
 
     // Transfer related actions
     KAction *deleteSelectedAction = actionCollection()->addAction("delete_selected_download");
-    deleteSelectedAction->setText(i18nc("delete selected transfer item", "Delete Selected"));
+    deleteSelectedAction->setText(i18nc("delete selected transfer item", "Remove Selected"));
     deleteSelectedAction->setIcon(KIcon("edit-delete"));
     deleteSelectedAction->setShortcuts(KShortcut("Del"));
     connect(deleteSelectedAction, SIGNAL(triggered()), SLOT(slotDeleteSelected()));
 
     KAction *deleteAllFinishedAction = actionCollection()->addAction("delete_all_finished");
-    deleteAllFinishedAction->setText(i18nc("delete all finished transfers", "Delete All Finished"));
+    deleteAllFinishedAction->setText(i18nc("delete all finished transfers", "Remove All Finished"));
     deleteAllFinishedAction->setIcon(KIcon("edit-clear-list"));
     connect(deleteAllFinishedAction, SIGNAL(triggered()), SLOT(slotDeleteFinished()));
 
@@ -274,6 +282,14 @@ void MainWindow::setupActions()
     stopActionMenu->addAction(stopSelectedAction);
     connect(stopActionMenu, SIGNAL(triggered()), SLOT(slotStopDownload()));
 
+    KActionMenu *deleteActionMenu = new KActionMenu(KIcon("edit-delete"), i18n("Remove"),
+                                                    actionCollection());    
+    actionCollection()->addAction("delete_menu", deleteActionMenu);
+    deleteActionMenu->setDelayed(true);
+    deleteActionMenu->addAction(deleteSelectedAction);
+    deleteActionMenu->addAction(deleteAllFinishedAction);
+    connect(deleteActionMenu, SIGNAL(triggered()), SLOT(slotDeleteSelected()));
+    
     KAction *openDestAction = actionCollection()->addAction("transfer_open_dest");
     openDestAction->setText(i18n("Open Destination"));
     openDestAction->setIcon(KIcon("document-open"));
@@ -284,9 +300,8 @@ void MainWindow::setupActions()
     openFileAction->setIcon(KIcon("document-open"));
     connect(openFileAction, SIGNAL(triggered()), SLOT(slotTransfersOpenFile()));
 
-    KAction *showDetailsAction = actionCollection()->addAction("transfer_show_details");
-    showDetailsAction->setText(i18n("Show Details"));
-    showDetailsAction->setIcon(KIcon("document-properties"));
+    KAction *showDetailsAction = new KToggleAction(KIcon("document-properties"), i18n("Show Details"), actionCollection());
+    actionCollection()->addAction("transfer_show_details", showDetailsAction);
     connect(showDetailsAction, SIGNAL(triggered()), SLOT(slotTransfersShowDetails()));
 
     KAction *copyUrlAction = actionCollection()->addAction("transfer_copy_source_url");
@@ -294,12 +309,6 @@ void MainWindow::setupActions()
     copyUrlAction->setIcon(KIcon("edit-copy"));
     connect(copyUrlAction, SIGNAL(triggered()), SLOT(slotTransfersCopySourceUrl()));
 
-    KToggleAction *showDropTargetAction = new KToggleAction(KIcon("kget"),
-                                          i18n("Show Drop Target"), actionCollection());
-    actionCollection()->addAction("show_drop_target", showDropTargetAction);
-    showDropTargetAction->setChecked(Settings::showDropTarget());
-    connect(showDropTargetAction, SIGNAL(triggered()), SLOT(slotToggleDropTarget()));
-
     KAction *transferHistoryAction = actionCollection()->addAction("transfer_history");
     transferHistoryAction->setText(i18n("&Transfer History"));
     transferHistoryAction->setIcon(KIcon("view-history"));
@@ -330,17 +339,9 @@ void MainWindow::slotDelayedInit()
     //Here we import the user's transfers.
     KGet::load( KStandardDirs::locateLocal("appdata", "transfers.kgt") );
 
-#ifdef HAVE_KNOTIFICATIONITEM
     if(Settings::enableSystemTray()) {
         m_dock = new Tray(this);
     }
-#else
-    m_dock = new Tray(this);
-
-    if(Settings::enableSystemTray()) {
-        m_dock->show();
-    }
-#endif
 
     // enable dropping
     setAcceptDrops(true);
@@ -370,7 +371,7 @@ void MainWindow::slotDelayedInit()
             slotKonquerorIntegration(true);
         }
 
-        m_drop->setDropTargetVisible(true);
+        m_drop->setDropTargetVisible(false);
 
         // reset the FirstRun config option
         Settings::setFirstRun(false);
@@ -386,9 +387,6 @@ void MainWindow::slotDelayedInit()
     if ( Settings::autoPaste() )
         clipboardTimer->start(1000);
 
-    // kget kuiserver integration
-    KGet::reloadKJobs();
-
     if (Settings::webinterfaceEnabled())
         m_webinterface = new HttpServer(this);
 
@@ -402,25 +400,39 @@ void MainWindow::slotDelayedInit()
         KGet::setGlobalDownloadLimit(0);
         KGet::setGlobalUploadLimit(0);
     }
+
+    connect(KGet::model(), SIGNAL(transferAddedEvent(TransferHandler *, TransferGroupHandler *)), this, SLOT(slotUpdateTitlePercent()));
+    connect(KGet::model(), SIGNAL(transferRemovedEvent(TransferHandler *, TransferGroupHandler *)), this, SLOT(slotUpdateTitlePercent()));
+    connect(KGet::model(), SIGNAL(transfersChangedEvent(QMap<TransferHandler *, Transfer::ChangesFlags>)), 
+                           SLOT(slotTransfersChanged(QMap<TransferHandler *, Transfer::ChangesFlags>)));
+    connect(KGet::model(), SIGNAL(groupsChangedEvent(QMap<TransferGroupHandler *, TransferGroup::ChangesFlags>)),
+                           SLOT(slotGroupsChanged(QMap<TransferGroupHandler *, TransferGroup::ChangesFlags>)));
+
+#ifdef DEBUG
+    if (m_doTesting)
+    {
+        // Unit testing
+        TestKGet unitTest;
+        QTest::qExec(&unitTest);
+    }
+#endif
 }
 
 void MainWindow::slotToggleDropTarget()
 {
-    actionCollection()->action("show_drop_target")->setChecked(!m_drop->isVisible());
-
     m_drop->setDropTargetVisible(!m_drop->isVisible());
 }
 
 void MainWindow::slotNewTransfer()
 {
-    NewTransferDialog::instance(this)->showDialog();
+    NewTransferDialog::showNewTransferDialog(KUrl(), this);
 }
 
 void MainWindow::slotImportTransfers()
 {
     QString filename = KFileDialog::getOpenFileName(KUrl(),
-                                                    "*.kgt *.metalink *.torrent|" + i18n("All Openable Files") +
-                                                    " (*.kgt *.metalink *.torrent)", this, i18n("Open File"));
+                                                    "*.kgt *.metalink *.meta4 *.torrent|" + i18n("All Openable Files") +
+                                                    " (*.kgt *.metalink *.meta4 *.torrent)", this, i18n("Open File"));
 
     if(filename.endsWith(QLatin1String(".kgt")))
     {
@@ -442,6 +454,46 @@ void MainWindow::slotUpdateTitlePercent()
     }
 }
 
+void MainWindow::slotTransfersChanged(QMap<TransferHandler*, Transfer::ChangesFlags> transfers)
+{
+    QMapIterator<TransferHandler*, Transfer::ChangesFlags> it(transfers);
+    
+    //QList<TransferHandler *> finishedTransfers;
+    bool update = false;
+    
+    while (it.hasNext()) {
+        it.next();
+        
+        //TransferHandler * transfer = it.key();
+        Transfer::ChangesFlags transferFlags = it.value();
+
+        if (transferFlags & Transfer::Tc_Percent || transferFlags & Transfer::Tc_Status) {
+            update = true;
+            break;
+        }
+        
+//         kDebug(5001) << it.key() << ": " << it.value() << endl;
+    }
+    
+    if (update)
+        slotUpdateTitlePercent();
+}
+
+void MainWindow::slotGroupsChanged(QMap<TransferGroupHandler*, TransferGroup::ChangesFlags> groups)
+{
+    bool update = false;
+    foreach (TransferGroup::ChangesFlags groupFlags, groups)
+    {
+        if (groupFlags & TransferGroup::Gc_Percent)
+        {
+            update = true;
+            break;
+        }
+    }
+    if (update)
+        slotUpdateTitlePercent();
+}
+
 void MainWindow::slotQuit()
 {
     if (KGet::schedulerRunning()) {
@@ -487,13 +539,16 @@ void MainWindow::slotExportPlainTransfers()
     exportTransfers(true);
 }
 
+void MainWindow::slotCreateMetalink()
+{
+    MetalinkCreator *dialog = new MetalinkCreator(this);
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+    dialog->show();
+}
+
 void MainWindow::slotDeleteGroup()
 {
-    foreach(TransferGroupHandler * it, KGet::selectedTransferGroups())
-    {
-        it->stop();
-        KGet::delGroup(it->name());
-    }
+    KGet::delGroups(KGet::selectedTransferGroups());
 }
 
 void MainWindow::slotRenameGroup()
@@ -558,6 +613,11 @@ void MainWindow::slotStopDownload()
 void MainWindow::slotStopAllDownload()
 {
     KGet::setSchedulerRunning(false);
+    
+    // This line ensures that each transfer is stopped. In the handler class
+    // the policy of the transfer will be correctly setted to None
+    foreach (TransferHandler * it, KGet::allTransfers())
+        it->stop();
 }
 
 void MainWindow::slotStopSelectedDownload()
@@ -687,10 +747,8 @@ void MainWindow::slotNewConfig()
     // parsed often by the code. When clicking Ok or Apply of
     // PreferencesDialog, this function is called.
 
-    m_viewsContainer->setExpandableDetails(Settings::showExpandableTransferDetails());
     m_drop->setDropTargetVisible(Settings::showDropTarget(), false);
 
-#ifdef HAVE_KNOTIFICATIONITEM
     if(Settings::enableSystemTray() && !m_dock)
     {
         m_dock = new Tray(this);
@@ -701,17 +759,10 @@ void MainWindow::slotNewConfig()
         delete m_dock;
         m_dock = 0;
     }
-#else
-    m_dock->setVisible(Settings::enableSystemTray());
-    if(!Settings::enableSystemTray())
-        setVisible(true);
-#endif
 
     slotKonquerorIntegration(Settings::konquerorIntegration());
     m_konquerorIntegration->setChecked(Settings::konquerorIntegration());
 
-    KGet::reloadKJobs();
-
     if (Settings::autoPaste())
         clipboardTimer->start(1000);
     else
@@ -753,18 +804,17 @@ void MainWindow::slotToggleAutoPaste()
 
 void MainWindow::slotCheckClipboard()
 {
-    QString clipData = QApplication::clipboard()->text( QClipboard::Clipboard ).trimmed();
+    const QString clipData = QApplication::clipboard()->text( QClipboard::Clipboard ).trimmed();
 
-    if (clipData != lastClipboard)
-    {
+    if (clipData != lastClipboard) {
         lastClipboard = clipData;
         if (lastClipboard.isEmpty())
             return;
 
-        KUrl url = KUrl(lastClipboard);
-
-        if (url.isValid() && !url.protocol().isEmpty() && url.hasPath() && !url.isLocalFile())
+        const KUrl url = KUrl(lastClipboard);
+        if (url.isValid() && !url.protocol().isEmpty() && url.hasPath() && url.hasHost() && !url.isLocalFile()) {
             KGet::addTransfer( url );
+        }
     }
 }
 
@@ -773,7 +823,7 @@ void MainWindow::slotTrayKonquerorIntegration(bool enable)
     slotKonquerorIntegration(enable);
     if (!enable && Settings::konquerorIntegration() && !Settings::expertMode())
     {
-        KGet::showNotification(this, KNotification::Notification,
+        KGet::showNotification(this, "notification",
                                      i18n("KGet has been temporarily disabled as download manager for Konqueror. "
             "If you want to disable it forever, go to Settings->Advanced and disable \"Use "
             "as download manager for Konqueror\"."),
@@ -807,19 +857,8 @@ void MainWindow::setSystemTrayDownloading(bool running)
 {
     kDebug(5001);
 
-#ifdef HAVE_KNOTIFICATIONITEM
     if (m_dock)
         m_dock->setDownloading(running);
-#else
-    m_dock->setDownloading(running);
-#endif
-}
-
-void MainWindow::importLinks(const QList <QString> &links)
-{
-    KGetLinkView *link_view = new KGetLinkView(this);
-    link_view->setLinks(links);
-    link_view->show();
 }
 
 void MainWindow::slotTransferHistory()
@@ -834,8 +873,9 @@ void MainWindow::slotTransferGroupSettings()
     QList<TransferGroupHandler*> list = KGet::selectedTransferGroups();
     foreach(TransferGroupHandler* group, list)
     {
-        GroupSettingsDialog *settings = new GroupSettingsDialog(this, group);
+        QPointer<GroupSettingsDialog> settings = new GroupSettingsDialog(this, group);
         settings->exec();
+        delete settings;
     }
 }
 
@@ -845,8 +885,9 @@ void MainWindow::slotTransferSettings()
     QList<TransferHandler*> list = KGet::selectedTransfers();
     foreach(TransferHandler* transfer, list)
     {
-        TransferSettingsDialog *settings = new TransferSettingsDialog(this, transfer);
+        QPointer<TransferSettingsDialog> settings = new TransferSettingsDialog(this, transfer);
         settings->exec();
+        delete settings;
     }
 }
 
@@ -909,7 +950,7 @@ void MainWindow::dropEvent(QDropEvent * event)
                                        KGuiItem(i18n("&Load transfer list"), KIcon("list-add")), KStandardGuiItem::cancel());
 
             if (msgBoxResult == 3) //Download
-                NewTransferDialog::instance(this)->showDialog(list.first().url());
+                NewTransferDialog::showNewTransferDialog(list.first().url(), this);
             if (msgBoxResult == 4) //Load
                 KGet::load(list.first().url());
         }
@@ -918,129 +959,16 @@ void MainWindow::dropEvent(QDropEvent * event)
             if (list.count() == 1)
             {
                 str = event->mimeData()->text();
-                NewTransferDialog::instance(this)->showDialog(str);
+                NewTransferDialog::showNewTransferDialog(str, this);
             }
             else
-                NewTransferDialog::instance(this)->showDialog(list);
+                NewTransferDialog::showNewTransferDialog(list, this);
         }
     }
     else
     {
-        NewTransferDialog::instance(this)->showDialog();
+        NewTransferDialog::showNewTransferDialog(KUrl(), this);
     }
 }
 
-
-/** DBUS interface */
-
-void MainWindow::addTransfer(const QString& src, const QString& dest, bool start)
-{
-    // split src for the case it is a QStringList (e.g. from konqueror plugin)
-    KGet::addTransfer(src.split(';'), dest, QString(), start);
-}
-
-void MainWindow::showNewTransferDialog(const QStringList &urls)
-{
-    NewTransferDialog::instance(this)->showDialog(urls);
-}
-
-bool MainWindow::dropTargetVisible() const
-{
-    return m_drop->isVisible();
-}
-
-void MainWindow::setDropTargetVisible( bool setVisible )
-{
-    if ( setVisible != Settings::showDropTarget() )
-        m_drop->setDropTargetVisible( setVisible );
-}
-
-void MainWindow::setOfflineMode( bool offline )
-{
-    KGet::setSchedulerRunning(offline);
-}
-
-bool MainWindow::offlineMode() const
-{
-    return !KGet::schedulerRunning();
-}
-
-QVariantMap MainWindow::transfers() const
-{
-    return m_dbusModelObserver->transfers();
-}
-
-int MainWindow::transfersSpeed() const
-{
-    return m_dbusModelObserver->transfersSpeed();
-}
-
-MainWindowModelObserver::MainWindowModelObserver(MainWindow *window) 
-    : QObject(window)
-{
-    m_window = window;
-    m_groupObserver = new MainWindowGroupObserver(window);
-}
-
-void MainWindowModelObserver::addedTransferGroupEvent(TransferGroupHandler *group)
-{
-    group->addObserver(m_groupObserver);
-}
-
-void MainWindowModelObserver::removedTransferGroupEvent(TransferGroupHandler *group)
-{
-    group->delObserver(m_groupObserver);
-}
-
-MainWindowGroupObserver::MainWindowGroupObserver(MainWindow *window) 
-    : QObject(window)
-{
-    m_window = window;
-    m_transferObserver = new MainWindowTransferObserver(window);
-}
-
-void MainWindowGroupObserver::groupChangedEvent(TransferGroupHandler * group)
-{
-    TransferGroupHandler::ChangesFlags transferFlags = group->changesFlags(this);
-    if (transferFlags & TransferGroup::Gc_Percent) {
-        m_window->slotUpdateTitlePercent();
-    }
-    group->resetChangesFlags(this);
-}
-
-void MainWindowGroupObserver::addedTransferEvent(TransferHandler * transfer, TransferHandler * after)
-{ 
-    Q_UNUSED(after);
-    transfer->addObserver(m_transferObserver);
-}
-
-void MainWindowGroupObserver::removedTransferEvent(TransferHandler * transfer)
-{
-    transfer->delObserver(m_transferObserver);
-
-    m_window->slotUpdateTitlePercent();
-}
-
-/**
-void MainWindowGroupObserver::movedTransferEvent(TransferHandler * transfer, TransferHandler * after)
-{
-    Q_UNUSED(transfer);
-    Q_UNUSED(after);
-}**/
-
-MainWindowTransferObserver::MainWindowTransferObserver(MainWindow *window)
-    : QObject(window)
-{
-    m_window = window;
-}
-
-void MainWindowTransferObserver::transferChangedEvent(TransferHandler *transfer)
-{
-    TransferHandler::ChangesFlags transferFlags = transfer->changesFlags(this);
-    if (transferFlags & Transfer::Tc_Percent || transferFlags & Transfer::Tc_Status) {
-        m_window->slotUpdateTitlePercent();
-    }
-    transfer->resetChangesFlags(this);
-}
-
 #include "mainwindow.moc"
