@@ -129,6 +129,16 @@ void Metalink::metalinkInit(const KUrl &src, const QByteArray &data)
         return;
     }
 
+    //offers a dialog to download the newest version of a dynamic metalink
+     if ((m_source.isLocalFile() || !justDownloaded) && m_metalink.dynamic) {
+        if (KMessageBox::questionYesNo(0, i18n("A newer version of this Metalink might exist, do you want to download it?"),
+                                       i18n("Redownload Metalink")) == KMessageBox::Yes) {
+            m_localMetalinkLocation.clear();
+            m_source = m_metalink.origin;
+            return;
+        }
+    }
+
     QList<KGetMetalink::File>::const_iterator it;
     QList<KGetMetalink::File>::const_iterator itEnd = m_metalink.files.files.constEnd();
     m_totalSize = 0;
@@ -157,6 +167,7 @@ void Metalink::metalinkInit(const KUrl &src, const QByteArray &data)
 
 //TODO compare available file size (<size>) with the sizes of the server while downloading?
 
+        connect(dataFactory, SIGNAL(capabilitiesChanged()), this, SLOT(slotUpdateCapabilities()));
         connect(dataFactory, SIGNAL(totalSize(KIO::filesize_t)), this, SLOT(totalSizeChanged(KIO::filesize_t)));
         connect(dataFactory, SIGNAL(processedSize(KIO::filesize_t)), this, SLOT(processedSizeChanged()));
         connect(dataFactory, SIGNAL(speed(ulong)), this, SLOT(speedChanged()));
@@ -212,6 +223,7 @@ void Metalink::metalinkInit(const KUrl &src, const QByteArray &data)
     }
 
     m_ready = !m_dataSourceFactory.isEmpty();
+    slotUpdateCapabilities();
 
     //the metalink-file has just been downloaded, so ask the user to choose the files that
     // should be downloaded
@@ -319,11 +331,6 @@ void Metalink::stop()
     }
 }
 
-bool Metalink::isResumable() const
-{
-    return true;
-}
-
 void Metalink::totalSizeChanged(KIO::filesize_t size)
 {
     m_totalSize = 0;
@@ -603,6 +610,7 @@ void Metalink::load(const QDomElement *element)
 
         DataSourceFactory *file = new DataSourceFactory(this);
         file->load(&factory);
+        connect(file, SIGNAL(capabilitiesChanged()), this, SLOT(slotUpdateCapabilities()));
         connect(file, SIGNAL(totalSize(KIO::filesize_t)), this, SLOT(totalSizeChanged(KIO::filesize_t)));
         connect(file, SIGNAL(processedSize(KIO::filesize_t)), this, SLOT(processedSizeChanged()));
         connect(file, SIGNAL(speed(ulong)), this, SLOT(speedChanged()));
@@ -623,6 +631,7 @@ void Metalink::load(const QDomElement *element)
         }
     }
     m_ready = !m_dataSourceFactory.isEmpty();
+    slotUpdateCapabilities();
 }
 
 void Metalink::save(const QDomElement &element)
@@ -857,4 +866,23 @@ void Metalink::setAvailableMirrors(const KUrl &file, const QHash<KUrl, QPair<boo
     m_dataSourceFactory[file]->setMirrors(mirrors);
 }
 
+void Metalink::slotUpdateCapabilities()
+{
+    Capabilities oldCap = capabilities();
+    Capabilities newCap = 0;
+    foreach (DataSourceFactory *file, m_dataSourceFactory) {
+        if (file->doDownload()) {//FIXME when a download did not start yet it should be moveable!!//FIXME why not working, when only two connections?
+            if (newCap) {
+                newCap &= file->capabilities();
+            } else {
+                newCap = file->capabilities();
+            }
+        }
+    }
+
+    if (newCap != oldCap) {
+        setCapabilities(newCap);
+    }
+}
+
 #include "metalink.moc"
