@@ -1,40 +1,42 @@
-/***************************************************************************
-                          texteffectplugin.cpp  -  description
-                             -------------------
-    begin                : jeu nov 14 2002
-    copyright            : (C) 2002 by Olivier Goffart
-    email                : ogoffart @ kde.org
- ***************************************************************************/
+/*
+    texteffectplugin.cpp  -  description
+    
+    Copyright (c) 2002      by Olivier Goffart <ogoffart@kde.org>
 
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+    Kopete    (c) 2002-2007 by the Kopete developers <kopete-devel@kde.org>
+
+    ***************************************************************************
+    *                                                                         *
+    *   This program is free software; you can redistribute it and/or modify  *
+    *   it under the terms of the GNU General Public License as published by  *
+    *   the Free Software Foundation; either version 2 of the License, or     *
+    *   (at your option) any later version.                                   *
+    *                                                                         *
+    ***************************************************************************
+*/
 
 #include <stdlib.h>
 
 #include <kdebug.h>
 #include <kgenericfactory.h>
+#include <QColor>
 
 #include "kopetechatsessionmanager.h"
 
 #include "texteffectplugin.h"
 #include "texteffectconfig.h"
 
-typedef KGenericFactory<TextEffectPlugin> TextEffectPluginFactory;
-K_EXPORT_COMPONENT_FACTORY( kopete_texteffect, TextEffectPluginFactory( "kopete_texteffect" )  )
+K_PLUGIN_FACTORY(TextEffectPluginFactory, registerPlugin<TextEffectPlugin>();)
+K_EXPORT_PLUGIN(TextEffectPluginFactory( "kopete_texteffect" ))
 
-TextEffectPlugin::TextEffectPlugin( QObject *parent, const char *name, const QStringList &/*args*/ )
-: Kopete::Plugin( TextEffectPluginFactory::instance(), parent, name )
+TextEffectPlugin::TextEffectPlugin( QObject *parent, const QVariantList &/*args*/ )
+: Kopete::Plugin( TextEffectPluginFactory::componentData(), parent )
 {
 	if( !pluginStatic_ )
 		pluginStatic_=this;
 
 	m_config = new TextEffectConfig;
+	m_config->load();
 
 	connect ( this , SIGNAL( settingsChanged() ) , this , SLOT( slotSettingsChanged() ) );
 
@@ -71,10 +73,10 @@ void TextEffectPlugin::slotOutgoingMessage( Kopete::Message& msg )
 		QString original=msg.plainBody();
 		QString resultat;
 
-		unsigned int c=0;
+		int c=0;
 		bool wavein=false;
 
-		for(unsigned int f=0;f<original.length();f++)
+		for(int f=0;f<original.length();f++)
 		{
 			QChar x=original[f];
 			if(f==0 || m_config->colorChar() || (m_config->colorWords() && x==' ' ))
@@ -93,7 +95,7 @@ void TextEffectPlugin::slotOutgoingMessage( Kopete::Message& msg )
 				}
 				resultat+="\">";
 			}
-			switch (x.latin1())
+			switch (x.toLatin1())
 			{
 				case '>':
 					resultat+="&gt;";
@@ -110,55 +112,55 @@ void TextEffectPlugin::slotOutgoingMessage( Kopete::Message& msg )
 				case 'A':
 					if(m_config->lamer())
 					{
-						resultat+="4";
+						resultat+='4';
 						break;
 					} //else, go to the default,  all other case have this check
 				case 'e':
 				case 'E':
 					if(m_config->lamer())
 					{
-						resultat+="3";
+						resultat+='3';
 						break;
 					}//else, go to the default,  all other case have this check
 				case 'i':
 				case 'I':
 					if(m_config->lamer())
 					{
-						resultat+="1";
+						resultat+='1';
 						break;
 					}//else, go to the default,  all other case have this check
 				case 'l':
 				case 'L':
 					if(m_config->lamer())
 					{
-						resultat+="|";
+						resultat+='|';
 						break;
 					}//else, go to the default,  all other case have this check
 				case 't':
 				case 'T':
 					if(m_config->lamer())
 					{
-						resultat+="7";
+						resultat+='7';
 						break;
 					}//else, go to the default,  all other case have this check
 				case 's':
 				case 'S':
 					if(m_config->lamer())
 					{
-						resultat+="5";
+						resultat+='5';
 						break;
 					}//else, go to the default,  all other case have this check
 				case 'o':
 				case 'O':
 					if(m_config->lamer())
 					{
-						resultat+="0";
+						resultat+='0';
 						break;
 					}//else, go to the default,  all other case have this check
 				default:
 					if(m_config->waves())
 					{
-						resultat+= wavein ? x.lower() : x.upper();
+						resultat+= wavein ? x.toLower() : x.toUpper();
 						wavein=!wavein;
 					}
 					else
@@ -168,7 +170,7 @@ void TextEffectPlugin::slotOutgoingMessage( Kopete::Message& msg )
 		}
 		if( m_config->colorChar() || m_config->colorWords() )
 			resultat+="</font>";
-		msg.setBody(resultat,Kopete::Message::RichText);
+		msg.setHtmlBody(resultat);
 	}
 
 	if(m_config->colorLines())
@@ -184,7 +186,7 @@ void TextEffectPlugin::slotOutgoingMessage( Kopete::Message& msg )
 				last_color=0;
 		}
 
-		msg.setFg(QColor (colors[last_color]));
+		msg.setForegroundColor(QColor (colors[last_color]));
 	}
 }
 
