@@ -4,7 +4,7 @@
     Copyright (c) 2006      Novell, Inc	 	 	 http://www.opensuse.org
     Copyright (c) 2004      SUSE Linux AG	 	 http://www.suse.com
     
-    Based on Iris, Copyright (C) 2003  Justin Karneges
+    Based on Iris, Copyright (C) 2003  Justin Karneges <justin@affinix.com>
     encode_method from Gaim src/protocols/novell/nmconn.c
     Copyright (c) 2004 Novell, Inc. All Rights Reserved
     
@@ -146,9 +146,8 @@ public:
 };
 
 ClientStream::ClientStream(Connector *conn, TLSHandler *tlsHandler, QObject *parent)
-:Stream(parent)
+:Stream(parent), d(new Private())
 {
-	d = new Private;
 	d->mode = Client;
 	d->conn = conn;
 	connect( d->conn, SIGNAL(connected()), SLOT(cr_connected()) );
